---

title: About Us
description: This is the page description

---


<div class="wrapper">
    <section class="container">
        <h3 class="title">{{ __page__.title }}</h3>
        <p>
            This is the about-us page
        </p>
        <a href="{{ get_url_for('index') }}">Home</a>

        {% include "content/more-details.md" %}
        
    </section>
</div>
