---
title: Contact Us
description: This is the page description
---

<div class="wrapper">
    <section class="container">
        <h3 class="title">{{ this.title }}</h3>
        <p>
          <form>
            <fieldset>
              <label for="nameField">Name</label>
              <input type="text" placeholder="Name" id="nameField">
              <label for="emailField">Email</label>
              <input type="text" placeholder="Email" id="emailField">
              <label for="commentField">Comment</label>
              <textarea placeholder="" id="commentField"></textarea>
              <div class="float-right">
                <input type="checkbox" id="confirmField">
                <label class="label-inline" for="confirmField">Send a copy to yourself</label>
              </div>
              <input class="button-primary" type="submit" value="Send">
            </fieldset>
          </form>
        </p>
        <a href="{{ get_url_for('index') }}"> Home</a>
    </section>
</div>
