
"""
This the about file
"""

__all__ = [
    "__title__",
    "__summary__",
    "__uri__",
    "__version__",
    "__author__",
    "__email__",
    "__license__",
    "__copyright__",
    "__long_description__"
]

__title__ = "SSG"
__version__ = "0.1.13"
__summary__ = "SSG is an elegant and modern static site generator for the common folks!"
__uri__ = "https://github.com/mardix/ssg"
__author__ = "Mardix"
__email__ = "mardix@illybee.com"
__license__ = "MIT"
__copyright__ = "(c) 2023 %s" % __author__
__long_description__ = """
SSG

SSG is an elegant static site generator!

Features

- HTML
- Markdown
- Single File Component

"""